<!DOCTYPE html>
<html>
    <head>
        <meta charset="utf-8" />
        <meta name="viewport" content="width=device-width" />
        <title>Base html</title>
    </head>
    <body>
        Folder to place app template
    </body>
</html>
